#ifndef SPI_LCD_H
#define SPI_LCD_H
#endif

/*********************************************************

Author: Hubert Gschwandtner <hubert@schorsch.at> www.schorsch.at
Basic routines for interfacing a LCD Type VLGEM1021-03 
Compatible with HB24208 / HB24209 
LCD-Controller KS0074 or compatible
This library is based on the HD44780 LCD-Lib of P.Fleury
Adapted to the SPI-Interface and the KS0074 Controller
This routine uses SPI-Mode 0
Version 1.0.0 

Used controller: ATMega8 
Used pin : PortB 2 - 5

Note: Add a 22k resitor as a pull up to high to the chip select pin
	of the LCD to avoid problems on using the SPI for 
	in circuit programming the controller.
*********************************************************/

#include <inttypes.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>

/* User section - adapt to your system */

#define print_string_from_flash		1						/* activ 1 */
#define Store_individual_character_from_flash_on_cgram	0   /* activ 1 */
#define print_string_from_EEPROM	1						/* activ 1 */
#define Store_individual_character_from_EEPROM_on_cgram	1	/* activ 1 */

#define SPI_PORT  			DDRB	/* SPI-PORT */
#define SS					2		/* SS Pin on SPI-Port */
#define MOSI				3		/* MOSI Pin on SPI-Port */
#define MISO				4		/* MISO Pin on SPI-Port */
#define	SCK					5		/* SCK Pin on SPI-Port */

#define LCD_LINES           2     /* number of visible lines of the display */
#define LCD_DISP_LENGTH    24     /* visibles characters per line of the display */
#define LCD_LINE_LENGTH  0x40     /**< internal line length of the display    */
#define LCD_START_LINE1  0x00     /**< DDRAM address of first char of line 1 */
#define LCD_START_LINE2  0x40     /**< DDRAM address of first char of line 2 */
#define LCD_START_LINE4  0x54     /**< DDRAM address of first char of line 4 */

/* End user section */


/* SPI-clock claculation - limits SPI-clock to 1MHz */
#define SPI_clock	(F_CPU / 0xcffff)
#if 	(SPI_clock) >16
#define SPR0x		0
#define SPR1x		1
#define	SPI2Xx		1
#elif	SPI_clock >8
#define	SPR0x		1
#define SPR1x		0
#define	SPI2Xx		0
#elif	SPI_clock >4
#define	SPR0x		1
#define SPR1x		0
#define	SPI2Xx		1
#elif	SPI_clock >2
#define	SPR0x		0
#define	SPR1x		0
#define	SPI2Xx		0
#else
#define	SPR0x		0
#define	SPR1x		0
#define	SPI2Xx		1
#endif

/* instruction register bit positions, see KS0074 data sheet */
#define LCD_CLR               0      /* DB0: clear display                  */
#define LCD_HOME              1      /* DB1: return to home position        */
#define LCD_ENTRY_MODE        2      /* DB2: set entry mode                 */
#define LCD_ENTRY_INC         1      /* DB1: 1=increment, 0=decrement     */
#define LCD_ENTRY_SHIFT       0      /* DB2: 1=display shift on           */
#define LCD_ON                3      /* DB3: turn lcd/cursor on             */
#define LCD_ON_DISPLAY        2      /* DB2: turn display on              */
#define LCD_ON_CURSOR         1      /* DB1: turn cursor on               */
#define LCD_ON_BLINK          0      /* DB0: blinking cursor ?          */
#define LCD_MOVE              4      /* DB4: move cursor/display            */
#define LCD_MOVE_DISP         3      /* DB3: move display (0-> cursor) ?  */
#define LCD_MOVE_RIGHT        2      /* DB2: move right (0-> left) ?      */
#define LCD_FUNCTION          5      /* DB5: function set                   */
#define LCD_FUNCTION_8BIT     4      /* DB4: set 8BIT mode (0->4BIT mode) */
#define LCD_FUNCTION_2LINES   3      /* DB3: two lines (0->one line)      */
#define LCD_FUNCTION_10DOTS   2      /* DB2: 5x10 font (0->5x7 font)      */
#define LCD_CGRAM             6      /* DB6: set CG RAM address             */
#define LCD_DDRAM             7      /* DB7: set DD RAM address             */
#define LCD_BUSY              7      /* DB7: LCD is busy                    */

/* set entry mode: display shift on/off, dec/inc cursor move direction */
#define LCD_ENTRY_DEC            0x04   /* display shift off, dec cursor move dir */
#define LCD_ENTRY_DEC_SHIFT      0x05   /* display shift on,  dec cursor move dir */
#define LCD_ENTRY_INC_           0x06   /* display shift off, inc cursor move dir */
#define LCD_ENTRY_INC_SHIFT      0x07   /* display shift on,  inc cursor move dir */

/* display on/off, cursor on/off, blinking char at cursor position */
#define LCD_DISP_OFF             0x08   /* display off                            */
#define LCD_DISP_ON              0x0C   /* display on, cursor off                 */
#define LCD_DISP_ON_BLINK        0x0D   /* display on, cursor off, blink char     */
#define LCD_DISP_ON_CURSOR       0x0E   /* display on, cursor on                  */
#define LCD_DISP_ON_CURSOR_BLINK 0x0F   /* display on, cursor on, blink char      */

/* move cursor/shift display */
#define LCD_MOVE_CURSOR_LEFT     0x10   /* move cursor left  (decrement)          */
#define LCD_MOVE_CURSOR_RIGHT    0x14   /* move cursor right (increment)          */
#define LCD_MOVE_DISP_LEFT       0x18   /* shift display left                     */
#define LCD_MOVE_DISP_RIGHT      0x1C   /* shift display right                    */

/* function set: set interface data length and number of display lines */
#define LCD_FUNCTION_4BIT_1LINE  0x20   /* 4-bit interface, single line, 5x7 dots */
#define LCD_FUNCTION_4BIT_2LINES 0x28   /* 4-bit interface, dual line,   5x7 dots */
#define LCD_FUNCTION_8BIT_1LINE  0x30   /* 8-bit interface, single line, 5x7 dots */
#define LCD_FUNCTION_8BIT_2LINES 0x38   /* 8-bit interface, dual line,   5x7 dots */	

/* Init for SPI and Display */
extern void lcd_init(unsigned char dispAttr);	

/* Send intern disp-command */
void slcd_command(unsigned char scmd);	

/* Send Disp-command */
extern void lcd_command(unsigned char cmd);		

/* Send Disp-Data */
extern void lcd_data(unsigned char data);		

/* Print character on LCD (no auto linefeed)*/
extern void lcd_putc(unsigned char c);

/* Clear screen, set cursor home */
extern void lcd_clrscr(void);			

/* print string on lcd (no auto linefeed) */
extern void lcd_puts(const  char *s);	

/* set cursor home */
extern void lcd_home(void);				

/* set cursor to position */
extern void lcd_gotoxy(uint8_t x, uint8_t y); 

#if print_string_from_flash
/* print string from flash */
extern void lcd_puts_p(const char *progmem_s);
#endif 

#if Store_individual_character_from_flash_on_cgram
/* Store individual character from flash on cgram */
extern void lcd_putc_cgram_p(const char *progmem_s, unsigned char _position);
#endif

#if print_string_from_EEPROM
/* print string from EEPROM */
extern void lcd_puts_eep(const unsigned char *eep); 
#endif

#if Store_individual_character_from_EEPROM_on_cgram
/* Store individual character from EEPROM on cgram */
extern void lcd_putc_cgram_eep(const unsigned char *eep, unsigned char _position);
#endif
